

TIME_FMT = "%Y-%m-%d %H:%M:%S"

WHOSHERE_VER = "0.1.20170828"
STAT_FILE = "/var/www/whoshere-status"
WWW_PATH = "/var/www"
LOG_DIR = "/var/log/whoshere"
PID_DIR = "/var/run/"
TIME_AWAY_DEFAULT = 660
CONFIG_FILE = "whoshere.ini"
TARGET_FILE = "mtargets.json"
IFACE = "eth0"  # eth0 em0
HTTP_PORT_NUMBER = 8088


SNIFF_TIMEOUT = 60 * 16
TIME_FMT = "%Y-%m-%d %H:%M:%S"

VERBOSE = 0
